Imports System.Drawing
Imports System.Windows.Forms
Imports System.Drawing.Drawing2D

<ToolboxBitmap(GetType(TabControl))> _
Public Class CustomTabControl : Inherits TabControl

#Region "    Constructor "

    Public Sub New()
        MyBase.New()

        '   Set painting to user paint if custom style selected
        If Me._DisplayManager.Equals(TabControlDisplayManager.Custom) Then
            Me.SetStyle(ControlStyles.UserPaint, True)
            Me.ItemSize = New Size(0, 15)
        End If

        '   Set transparency support and double buffering to reduce flicker
        Me.SetStyle(ControlStyles.SupportsTransparentBackColor, True)
        Me.SetStyle(ControlStyles.OptimizedDoubleBuffer, True)
        Me.ResizeRedraw = True

    End Sub

#End Region

#Region "    Variables "

    Private _DisplayManager As TabControlDisplayManager = TabControlDisplayManager.Custom

#End Region

#Region "    Properties "

    <System.ComponentModel.DefaultValue(GetType(TabControlDisplayManager), "Custom")> _
    Public Property DisplayManager() As TabControlDisplayManager
        Get
            Return Me._DisplayManager
        End Get
        Set(ByVal value As TabControlDisplayManager)
            If Me._DisplayManager <> value Then
                Me._DisplayManager = value

                '   Set the style and turn on user painting
                If Me._DisplayManager.Equals(TabControlDisplayManager.Custom) Then
                    Me.SetStyle(ControlStyles.UserPaint, True)
                    Me.ItemSize = New Size(0, 15)
                Else
                    Me.SetStyle(ControlStyles.UserPaint, False)
                    Me.ItemSize = New Size(0, 0)
                End If
            End If
        End Set
    End Property

#End Region

#Region "    Background painting "

    Protected Overrides Sub OnPaintBackground(ByVal pevent As System.Windows.Forms.PaintEventArgs)
        If Me.DesignMode Then
            '   If this is in the designer let's put a nice gradient on the background
            '   By default the tabcontrol has a fixed grey background.  Yuck!
            Dim backBrush As New LinearGradientBrush( _
                Me.Bounds, _
                SystemColors.ControlLightLight, _
                SystemColors.ControlLight, _
                Drawing2D.LinearGradientMode.Vertical)
            pevent.Graphics.FillRectangle(backBrush, Me.Bounds)
            backBrush.Dispose()
        Else
            '   At runtime we want a transparent background.
            '   So let's paint the containing control (there has to be one).

            Me.InvokePaintBackground(Me.Parent, pevent)
            Me.InvokePaint(Me.Parent, pevent)
        End If
    End Sub

#End Region

#Region "    Painting the tabs "

    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)

        '   Paint all the tabs
        '   The cliprectangle of the paint event will take care of 
        '   not repainting stuff that has not changed.
        Me.PaintAllTheTabs(e)

        '   paint a border round the pagebox
        '   We can't make the border disappear so have to do it like this.
        Me.PaintTheTabPageBorder(e)

        '   repaint the bit where the selected tab is
        Me.PaintTheSelectedTab(e)
    End Sub

    Private Sub PaintAllTheTabs(ByVal e As System.Windows.Forms.PaintEventArgs)
        If Me.TabCount > 0 Then
            For index As Integer = 0 To Me.TabCount - 1
                Me.PaintTab(e, index)
            Next
        End If
    End Sub

    Private Sub PaintTab(ByVal e As System.Windows.Forms.PaintEventArgs, ByVal index As Integer)

        Dim path As System.Drawing.Drawing2D.GraphicsPath = Me.GetPath(index)
        Me.PaintTabBackground(e.Graphics, index, path)
        Me.PaintTabBorder(e.Graphics, index, path)
        Me.PaintTabText(e.Graphics, index)

    End Sub

    Private Sub PaintTabBackground(ByVal graph As System.Drawing.Graphics, ByVal index As Integer, ByVal path As System.Drawing.Drawing2D.GraphicsPath)
        Dim rect As Rectangle = Me.GetTabRect(index)

        Dim buttonBrush As System.Drawing.Brush = _
            New System.Drawing.Drawing2D.LinearGradientBrush( _
                rect, _
                SystemColors.ControlLightLight, _
                SystemColors.ControlLight, _
                Drawing2D.LinearGradientMode.Vertical)

        If index = Me.SelectedIndex Then
            buttonBrush = New System.Drawing.SolidBrush(SystemColors.ControlLightLight)
        End If

        graph.FillPath(buttonBrush, path)
        buttonBrush.Dispose()

    End Sub

    Private Sub PaintTabBorder(ByVal graph As System.Drawing.Graphics, ByVal index As Integer, ByVal path As System.Drawing.Drawing2D.GraphicsPath)
        Dim borderPen As New Pen(SystemColors.ControlDark)
        If index = Me.SelectedIndex Then
            borderPen = New Pen(ThemedColors.ToolBorder)
        End If

        graph.DrawPath(borderPen, path)
        borderPen.Dispose()
    End Sub

    Private Sub PaintTabText(ByVal graph As System.Drawing.Graphics, ByVal index As Integer)
        Dim rect As Rectangle = Me.GetTabRect(index)
        Dim rect2 As New Rectangle(rect.Left + rect.Height + 2, rect.Top, rect.Width - rect.Height, rect.Height)
        If index > 0 Then rect2 = New Rectangle(rect.Left + 8, rect.Top, rect.Width - 6, rect.Height)

        Dim tabtext As String = Me.TabPages(index).Text

        Dim format As New System.Drawing.StringFormat()
        format.Alignment = StringAlignment.Near
        format.LineAlignment = StringAlignment.Center
        format.Trimming = StringTrimming.EllipsisCharacter

        Dim forebrush As New SolidBrush(Me.ForeColor)
        graph.DrawString(tabtext, Me.Font, forebrush, rect2, format)
        forebrush.Dispose()
    End Sub

    Private Sub PaintTheTabPageBorder(ByVal e As System.Windows.Forms.PaintEventArgs)
        If Me.TabCount > 0 Then

            Dim borderRect As Rectangle = Me.TabPages(0).Bounds
            borderRect.Inflate(1, 1)
            ControlPaint.DrawBorder(e.Graphics, borderRect, ThemedColors.ToolBorder, ButtonBorderStyle.Solid)

        End If
    End Sub

    Private Sub PaintTheSelectedTab(ByVal e As System.Windows.Forms.PaintEventArgs)
        Select Case Me.SelectedIndex
            Case -1
            Case 0
                Dim selrect As Rectangle = Me.GetTabRect(Me.SelectedIndex)
                Dim selrectRight As Integer = selrect.Right
                e.Graphics.DrawLine(SystemPens.ControlLightLight, selrect.Left + 2, selrect.Bottom + 1, selrectRight - 2, selrect.Bottom + 1)
            Case Else
                Dim selrect As Rectangle = Me.GetTabRect(Me.SelectedIndex)
                Dim selrectRight As Integer = selrect.Right
                e.Graphics.DrawLine(SystemPens.ControlLightLight, selrect.Left, selrect.Bottom + 1, selrectRight - 2, selrect.Bottom + 1)
        End Select
    End Sub

    Private Function GetPath(ByVal index As Integer) As System.Drawing.Drawing2D.GraphicsPath
        Dim path As New System.Drawing.Drawing2D.GraphicsPath()
        path.Reset()

        Dim rect As Rectangle = Me.GetTabRect(index)

        If index = 0 Then
            path.AddLine(rect.Left + 1, rect.Bottom + 1, rect.Left + rect.Height, rect.Top + 2)
            path.AddLine(rect.Left + rect.Height + 4, rect.Top, rect.Right - 3, rect.Top)
            path.AddLine(rect.Right - 1, rect.Top + 2, rect.Right - 1, rect.Bottom + 1)
        Else
            If index = Me.SelectedIndex Then
                path.AddLine(rect.Left + 1, rect.Top + 5, rect.Left + 4, rect.Top + 2)
                path.AddLine(rect.Left + 8, rect.Top, rect.Right - 3, rect.Top)
                path.AddLine(rect.Right - 1, rect.Top + 2, rect.Right - 1, rect.Bottom + 1)
                path.AddLine(rect.Right - 1, rect.Bottom + 1, rect.Left + 1, rect.Bottom + 1)
            Else
                path.AddLine(rect.Left, rect.Top + 6, rect.Left + 4, rect.Top + 2)
                path.AddLine(rect.Left + 8, rect.Top, rect.Right - 3, rect.Top)
                path.AddLine(rect.Right - 1, rect.Top + 2, rect.Right - 1, rect.Bottom + 1)
                path.AddLine(rect.Right - 1, rect.Bottom + 1, rect.Left, rect.Bottom + 1)
            End If
        End If
        Return path
    End Function

#End Region

    Public Enum TabControlDisplayManager As Integer
        [Default]
        Custom
    End Enum

End Class
